#!/opt/VRTSsfmh/bin/perl
#
#$Id$
#$Source$
# $Copyright (c) 2017 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other 
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
#

BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use strict;
use warnings;
use File::Spec qw(catdir curdir catfile);
use VRTS::OS;
use VRTS::Ini;
use VRTS::Util;
use VRTS::Paths;
use JSON;
use FileHandle;

# If OS = Solaris, and version is less than 10, then dont install this addon
if ( $^O =~ /Solaris/i ){
    open(HANDLE, "uname -r | cut -d. -f2 |");
    my $plat = <HANDLE>;
    close(HANDLE);
    chomp($plat);

    if ($plat < 10){
        print STDERR "Fabric addon is supported on Solaris SPARC 10 onwards.\n";
        exit 1;
    }
}

# Check environment for Add-on installation
#
my $install_dir = VRTS::Paths::get_path("InstallDir");
$install_dir = File::Spec->canonpath($install_dir);
if (!defined($install_dir)) {
    print("Cannot get Veritas InfoScale Operations Manager install directory\n");
}

# The addon build process will put actual version here
my $version = "7.4.2.0000";
my $addon_dir = VRTS::Paths::get_path("DeployStoreDir");
$addon_dir = File::Spec->catfile($addon_dir ,"VRTSsfmfabric-".$version);

my $mhbin = File::Spec->catdir($install_dir, "bin");
my $mhbin_perl = File::Spec->catfile($mhbin, "perl");
my $model_file = File::Spec->catdir($install_dir,"etc","tdf","sfm.model");

# Add-on installation sets the env variable "store" that points to the add-on
# top level dir for the given platform (in the spool dir)
my $store_dir = $ENV{Store};
$store_dir = File::Spec->canonpath($store_dir);
if (!defined($store_dir)) {
    print("Add-on spool directory not specified in environment\n");
}
$store_dir=~ s/\/\s*$//;


# Uncompress mh.tar which already contains files under correct structure
my $mh = File::Spec->catfile($store_dir,"mh.tar");
my $ret = VRTS::OS->ungzip_and_untar("$install_dir", "$mh");
if ($ret != 0) {
    #Unzip failed
    print STDERR "Failed to uncompress archive \"$mh\"\n";
    exit 1;
} else {
    print "Installed fabric discovery files\n";
}

# Read entries from fabric.sfm.model and merge them into sfm.model file
my $jsonobject = new JSON ();
my $org_hash;
my $fab_hash;

# Read original sfm.model
if (open(FILE, "< $model_file")) {
    my $buf = do { local $/; <FILE> };
    $org_hash = $jsonobject->decode($buf);
    close (FILE);
}

# Read fabric's sfm.model
my $fab_model_file = File::Spec->catfile($addon_dir, "fabric.sfm.model");
if (open(FILE, "< $fab_model_file")) {
    my $buf1 = do { local $/; <FILE> };
    $fab_hash = $jsonobject->decode($buf1);
    close (FILE);
}

# Add entries from fabric's sfm.model
if (defined($fab_hash)) {
    while (my($key,$value) = each(%{$fab_hash})) {
        if (!exists($org_hash->{$key})) {
            $org_hash->{$key} = $value;
        } else {
            # Merge the two values which are hashes themselves
            my $orig_val = $org_hash->{$key};
            my %union_val = (%$orig_val, %$value);
            $org_hash->{$key} = \%union_val;
        }
    }
}

# Write the updated hash to sfm.model
my $fd = new FileHandle $model_file, O_WRONLY|O_CREAT;
my $final_buf = JSON::to_json($org_hash, {pretty => 1, indent => 1});
print $fd "$final_buf\n";
close($fd);

# Generate constants by running tdf2constant.pl on fabric related tdfs
# The tdfs are common between different switch families, so just pick up BRCD
# The output constant pm files are generated under 'Constants' directory udner CWD
my $const_dir = File::Spec->catdir($install_dir, "agentlet", "VRTS");
if (!chdir($const_dir)) {
    print STDERR "Failed to change directory to \"$const_dir\"\n";
} else {
    my $tdf2cpl = File::Spec->catfile($mhbin, "tdf2constant.pl");
    my $tdfdir = File::Spec->catdir($install_dir, "etc", "tdf", "BRCD");

    print "Generating constants from \"$tdfdir\" into \"$const_dir\"\n";
    system("\"$mhbin_perl\" \"$tdf2cpl\" --dir \"$tdfdir\"");
    if ($?) {
        print STDERR "Failed to generate constants\n";
    } else {
        print "Successfully generated switch tdf constants\n";
    }
}

# Register fabric families
# Get domains to which this host is configured
my $sfm_conf = VRTS::OS->get_resolv();
my @doms;
my $result = VRTS::Ini->query_section($sfm_conf, "domains", \@doms);
if ($result != 0) {
    print STDERR "Failed to get domains from [$sfm_conf]\n";
    exit(1);
}
my $mhctl = File::Spec->catfile($mhbin,"mh_ctl.pl");
# Register only with sfm domain (we have ER & others as well).
foreach my $fam ("BRCD", "CISCO") {
    foreach my $dom (@doms) {
        next if ($dom !~ /^sfm:(.*)/);
        my $cmd = make_command($mhbin_perl, $mhctl, "--family", $fam,
                "--register", "--domain", $dom, "--schedule", "1440");
        system($cmd);
    }
    my $cmd2 = make_command($mhbin_perl, $mhctl, "--family", $fam, "--pause");
    system($cmd2);
}
# On non-Windows OS, the switch_driver.pl has to be an executable.
if ($^O !~ /Win32/i) {
    my $cmd = "chmod +x $mhbin/switch_driver.pl";
    system($cmd);    
}

# If the current host is MS, install ms.tar.gz under the CSInstallDir
my $csinstalldir = $install_dir;
# Replace ending VRTSsfmh by VRTSsfmcs and also remove ending '/' or '\' if any
$csinstalldir =~ s/VRTSsfmh[\/\\]?$/VRTSsfmcs/;
if (-d "$csinstalldir") {
    my $mstar = File::Spec->catfile($store_dir,"ms.tar.gz");
    $ret = VRTS::OS->ungzip_and_untar("$csinstalldir", "$mstar");
    if ($ret != 0) {
        #Unzip failed
        print STDERR "Failed to uncompress archive \"$mstar\"\n";
        exit 1;
    } else {
        print "Installed fabric online help files\n";
    }
}


print "Installed Fabric Add-on successfully";
exit(0);
